defmodule DecentralisedBookIndexWeb.UserLive.Index do
  use DecentralisedBookIndexWeb, :live_view

  alias DecentralisedBookIndex.Accounts.User

  @impl true
  def render(assigns) do
    ~H"""
    <.header>
      Listing Users
    </.header>

    <div class="pt-2">
      <.table
        id="users"
        rows={@users}
        row_click={fn {_id, user} -> JS.navigate(~p"/users/#{user}") end}
      >
        <:col :let={{_id, user}} label="Email">{user.email}</:col>

        <:col :let={{_id, user}} label="Role">{user.role}</:col>

        <:action :let={{_id, user}}>
          <div class="sr-only">
            <.link navigate={~p"/users/#{user}"}>Show</.link>
          </div>

          <.link patch={~p"/users/#{user}/edit"}>Edit Role</.link>
        </:action>
      </.table>
    </div>
    """
  end

  @impl true
  def mount(params, _session, socket) do
    search_query = Map.get(params, "query", "")
    sort_by = Map.get(params, "sort_by", "name")
    page_params = AshPhoenix.LiveView.page_from_params(params, 10)

    page = User.search!(
      search_query,
      page: page_params ++ [count: true],
      actor: socket.assigns.current_user
    )

    socket =
      socket
      |> assign(:sort_by, sort_by)
      |> assign(:search_query, search_query)
      |> assign(:page_params, page_params)
      |> assign(:page, page)
      |> assign(:users, user_row(page.results))
      |> assign(:params, params)
      |> apply_action(socket.assigns.live_action, params)

    {:ok, socket}
  end

  @impl true
  def handle_params(params, _url, socket) do
    {:noreply, apply_action(socket, socket.assigns.live_action, params)}
  end

  defp apply_action(socket, :index, _params) do
    socket
    |> assign(:page_title, "Listing Users")
  end

  defp user_row(users) do
    Enum.map(users, fn user -> {user.id, user} end)
  end
end
